

custom_targetfaces = {}
function init_custom_targetfaces() {

	custom_targetfaces["NRA_LR"] = {
		name: "example NRA LR",
		dist: 1000,
		dist_unit: "y",
		score: [10, "X"],
		board: {w: 72/INCH, h: 72/INCH, line: 20, score: 6},
		rings: [
			{diam: 10/INCH, color: "b", line: 10, score: "X"},
			{diam: 20/INCH, color: "b", line: 10, score: 10},
			{diam: 30/INCH, color: "b", line: 10, score: 9},
			{diam: 44/INCH, color: "b", line: 10, score: 8},
			{diam: 60/INCH, color: "w", line: 10, score: 7},
		],
	};

	
	
}


